# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class VCardName(UniversalBaseModel):
    """
    Structured name components.
    """

    family_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="familyName")] = pydantic.Field(
        default=None
    )
    """
    Last name or surname.
    """

    given_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="givenName")] = pydantic.Field(
        default=None
    )
    """
    First name or given name.
    """

    additional_names: typing_extensions.Annotated[
        typing.Optional[typing.List[str]], FieldMetadata(alias="additionalNames")
    ] = pydantic.Field(default=None)
    """
    Middle names or additional names.
    """

    honorific_prefixes: typing_extensions.Annotated[
        typing.Optional[typing.List[str]], FieldMetadata(alias="honorificPrefixes")
    ] = pydantic.Field(default=None)
    """
    Title prefixes like "Dr.", "Mr.", "Ms.".
    """

    honorific_suffixes: typing_extensions.Annotated[
        typing.Optional[typing.List[str]], FieldMetadata(alias="honorificSuffixes")
    ] = pydantic.Field(default=None)
    """
    Title suffixes like "Jr.", "Sr.", "PhD".
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.audience_count_only import AudienceCountOnly
from ..types.delete_audience_response import DeleteAudienceResponse
from ..types.error import Error
from .types.audiences_get_response import AudiencesGetResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAudiencesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        *,
        id: str,
        page: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AudiencesGetResponse]:
        """
        Retrieve an audience by ID with optional pagination.

        Parameters
        ----------
        id : str
            Audience ID. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`.

        page : typing.Optional[int]
            Page number. If provided with or without limit, returns paginated contacts.

        limit : typing.Optional[int]
            Items per page. If provided with or without page, returns paginated contacts.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AudiencesGetResponse]
            Successfully retrieved audience.
        """
        _response = self._client_wrapper.httpx_client.request(
            "audiences",
            method="GET",
            params={
                "id": id,
                "page": page,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AudiencesGetResponse,
                    parse_obj_as(
                        type_=AudiencesGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        contacts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AudienceCountOnly]:
        """
        Create a new audience with optional initial contacts. Phone numbers that don't exist will be auto-created as contacts.

        Parameters
        ----------
        name : str
            Audience name.

        description : typing.Optional[str]
            Audience description.

        contacts : typing.Optional[typing.Sequence[str]]
            Optional array of phone numbers (E.164 format) or contact IDs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AudienceCountOnly]
            Successfully created the audience.
        """
        _response = self._client_wrapper.httpx_client.request(
            "audiences",
            method="POST",
            json={
                "name": name,
                "description": description,
                "contacts": contacts,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AudienceCountOnly,
                    parse_obj_as(
                        type_=AudienceCountOnly,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteAudienceResponse]:
        """
        Permanently delete an audience and all its contact associations.

        Note: This will NOT delete the contacts themselves, only the audience and its memberships.

        Parameters
        ----------
        id : str
            Audience ID. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteAudienceResponse]
            Successfully deleted the audience.
        """
        _response = self._client_wrapper.httpx_client.request(
            "audiences",
            method="DELETE",
            params={
                "id": id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteAudienceResponse,
                    parse_obj_as(
                        type_=DeleteAudienceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        *,
        id: str,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AudienceCountOnly]:
        """
        Update audience metadata. This endpoint does NOT modify contacts.

        To add or remove contacts, use the [Add Contacts](/api-reference/audiences/add-contacts) or [Remove Contacts](/api-reference/audiences/remove-contacts) endpoints.

        Parameters
        ----------
        id : str
            Audience ID to update. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`.

        name : typing.Optional[str]
            New audience name.

        description : typing.Optional[str]
            New audience description.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AudienceCountOnly]
            Successfully updated the audience.
        """
        _response = self._client_wrapper.httpx_client.request(
            "audiences",
            method="PATCH",
            json={
                "id": id,
                "name": name,
                "description": description,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AudienceCountOnly,
                    parse_obj_as(
                        type_=AudienceCountOnly,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAudiencesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        *,
        id: str,
        page: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AudiencesGetResponse]:
        """
        Retrieve an audience by ID with optional pagination.

        Parameters
        ----------
        id : str
            Audience ID. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`.

        page : typing.Optional[int]
            Page number. If provided with or without limit, returns paginated contacts.

        limit : typing.Optional[int]
            Items per page. If provided with or without page, returns paginated contacts.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AudiencesGetResponse]
            Successfully retrieved audience.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "audiences",
            method="GET",
            params={
                "id": id,
                "page": page,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AudiencesGetResponse,
                    parse_obj_as(
                        type_=AudiencesGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        contacts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AudienceCountOnly]:
        """
        Create a new audience with optional initial contacts. Phone numbers that don't exist will be auto-created as contacts.

        Parameters
        ----------
        name : str
            Audience name.

        description : typing.Optional[str]
            Audience description.

        contacts : typing.Optional[typing.Sequence[str]]
            Optional array of phone numbers (E.164 format) or contact IDs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AudienceCountOnly]
            Successfully created the audience.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "audiences",
            method="POST",
            json={
                "name": name,
                "description": description,
                "contacts": contacts,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AudienceCountOnly,
                    parse_obj_as(
                        type_=AudienceCountOnly,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteAudienceResponse]:
        """
        Permanently delete an audience and all its contact associations.

        Note: This will NOT delete the contacts themselves, only the audience and its memberships.

        Parameters
        ----------
        id : str
            Audience ID. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteAudienceResponse]
            Successfully deleted the audience.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "audiences",
            method="DELETE",
            params={
                "id": id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteAudienceResponse,
                    parse_obj_as(
                        type_=DeleteAudienceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        *,
        id: str,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AudienceCountOnly]:
        """
        Update audience metadata. This endpoint does NOT modify contacts.

        To add or remove contacts, use the [Add Contacts](/api-reference/audiences/add-contacts) or [Remove Contacts](/api-reference/audiences/remove-contacts) endpoints.

        Parameters
        ----------
        id : str
            Audience ID to update. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`.

        name : typing.Optional[str]
            New audience name.

        description : typing.Optional[str]
            New audience description.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AudienceCountOnly]
            Successfully updated the audience.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "audiences",
            method="PATCH",
            json={
                "id": id,
                "name": name,
                "description": description,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AudienceCountOnly,
                    parse_obj_as(
                        type_=AudienceCountOnly,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_card_address_schema_type_item import VCardAddressSchemaTypeItem


class VcardAddress(UniversalBaseModel):
    country_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="countryName")] = (
        pydantic.Field(default=None)
    )
    """
    Country name.
    """

    extended_address: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="extendedAddress")] = (
        pydantic.Field(default=None)
    )
    """
    Extended address information (apartment, suite, etc.).
    """

    fulladdress: typing.Optional[str] = pydantic.Field(default=None)
    """
    Complete formatted address as a single string.
    """

    locality: typing.Optional[str] = pydantic.Field(default=None)
    """
    City or locality.
    """

    postal_code: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="postalCode")] = pydantic.Field(
        default=None
    )
    """
    ZIP code or postal code.
    """

    post_office_box: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="postOfficeBox")] = (
        pydantic.Field(default=None)
    )
    """
    P.O. Box number.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    State, province, or region.
    """

    street_address: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="streetAddress")] = (
        pydantic.Field(default=None)
    )
    """
    Street address or street name and number.
    """

    type: typing.Optional[typing.List[VCardAddressSchemaTypeItem]] = pydantic.Field(default=None)
    """
    Address type classifications.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .get_dlc_campaign_status_response_updates import GetDlcCampaignStatusResponseUpdates
from .profile_status_enum import ProfileStatusEnum


class DlcCampaignStatus(UniversalBaseModel):
    """
    Response for get DLC campaign status.
    """

    error: str = pydantic.Field()
    """
    Summary of errors related to the verification process. <br>
    
    These errors may request additional information or point out erroneous and/or missing fields.
    """

    id: str = pydantic.Field()
    """
    The unique identifier of the 10DLC campaign. This identifier is a string that always begins with the prefix `dlc_`, for example: `dlc_1234567890`.
    """

    status: ProfileStatusEnum = pydantic.Field()
    """
    Current review status of the DLC campaign. <br>
    
    `INCOMPLETE`: Not submitted.<br>
    `IN REVIEW`: Being reviewed by carriers.<br>
    `VERIFIED`: Approved and ready to send messages.<br>
    `FAILED`: Issues and errors related to the campaign's details. See [error](dlc-campaign#response.body.error) for these potential issues.
    """

    updates: GetDlcCampaignStatusResponseUpdates = pydantic.Field()
    """
    Status of local long code numbers that are being attached to this campaign. <br>
    
    Note that phone numbers can only be attached to `VERIFIED` campaigns via the [attach campaign endpoint](../phone-numbers/attach-campaign).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .company_entity_type_enum import CompanyEntityTypeEnum
from .company_sector_enum import CompanySectorEnum
from .company_type_enum import CompanyTypeEnum
from .optional_contacts import OptionalContacts


class OptionalBrandInfo(UniversalBaseModel):
    address: typing.Optional[str] = pydantic.Field(default=None)
    """
    Primary brand address where this brand is located.
    """

    contact: typing.Optional[OptionalContacts] = None
    dba: typing.Optional[str] = pydantic.Field(default=None)
    """
    "Doing Business As" name - the public name this brand operates under.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Brief description of what this brand does.
    """

    ein: typing.Optional[str] = pydantic.Field(default=None)
    """
    Employer Identification Number (EIN) assigned by the IRS.
    """

    email: typing.Optional[str] = pydantic.Field(default=None)
    """
    Main contact email address for this brand.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Legal name of the brand as registered.
    """

    sector: typing.Optional[CompanySectorEnum] = None
    type: typing.Optional[CompanyTypeEnum] = None
    entity_type: typing_extensions.Annotated[
        typing.Optional[CompanyEntityTypeEnum], FieldMetadata(alias="entityType")
    ] = pydantic.Field(default=None)
    """
    Legal entity type of the brand.
    """

    website: typing.Optional[str] = pydantic.Field(default=None)
    """
    Brand website URL.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DlcAssignmentStatusEnum = typing.Union[
    typing.Literal[
        "ASSIGNED", "FAILED_ASSIGNMENT", "FAILED_UNASSIGNMENT", "PENDING_ASSIGNMENT", "PENDING_UNASSIGNMENT"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawCampaignsClient, RawCampaignsClient

if typing.TYPE_CHECKING:
    from .dlc.client import AsyncDlcClient, DlcClient
    from .rcs.client import AsyncRcsClient, RcsClient
    from .toll_free.client import AsyncTollFreeClient, TollFreeClient


class CampaignsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCampaignsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._dlc: typing.Optional[DlcClient] = None
        self._toll_free: typing.Optional[TollFreeClient] = None
        self._rcs: typing.Optional[RcsClient] = None

    @property
    def with_raw_response(self) -> RawCampaignsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCampaignsClient
        """
        return self._raw_client

    @property
    def dlc(self):
        if self._dlc is None:
            from .dlc.client import DlcClient  # noqa: E402

            self._dlc = DlcClient(client_wrapper=self._client_wrapper)
        return self._dlc

    @property
    def toll_free(self):
        if self._toll_free is None:
            from .toll_free.client import TollFreeClient  # noqa: E402

            self._toll_free = TollFreeClient(client_wrapper=self._client_wrapper)
        return self._toll_free

    @property
    def rcs(self):
        if self._rcs is None:
            from .rcs.client import RcsClient  # noqa: E402

            self._rcs = RcsClient(client_wrapper=self._client_wrapper)
        return self._rcs


class AsyncCampaignsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCampaignsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._dlc: typing.Optional[AsyncDlcClient] = None
        self._toll_free: typing.Optional[AsyncTollFreeClient] = None
        self._rcs: typing.Optional[AsyncRcsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawCampaignsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCampaignsClient
        """
        return self._raw_client

    @property
    def dlc(self):
        if self._dlc is None:
            from .dlc.client import AsyncDlcClient  # noqa: E402

            self._dlc = AsyncDlcClient(client_wrapper=self._client_wrapper)
        return self._dlc

    @property
    def toll_free(self):
        if self._toll_free is None:
            from .toll_free.client import AsyncTollFreeClient  # noqa: E402

            self._toll_free = AsyncTollFreeClient(client_wrapper=self._client_wrapper)
        return self._toll_free

    @property
    def rcs(self):
        if self._rcs is None:
            from .rcs.client import AsyncRcsClient  # noqa: E402

            self._rcs = AsyncRcsClient(client_wrapper=self._client_wrapper)
        return self._rcs



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.rcs_capabilities_result import RcsCapabilitiesResult
from ..types.rcs_link_result import RcsLinkResult
from ..types.rcs_whitelist_response import RcsWhitelistResponse
from .raw_client import AsyncRawRcsClient, RawRcsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RcsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRcsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawRcsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRcsClient
        """
        return self._raw_client

    def get_capabilities(
        self, *, phone_numbers: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> RcsCapabilitiesResult:
        """
        Check RCS capabilities for one or more phone numbers.

        This endpoint allows you to verify which RCS features (cards, buttons, etc.) are supported
        on specific phone numbers before sending RCS messages to them.

        Parameters
        ----------
        phone_numbers : typing.Sequence[str]
            List of phone numbers to check RCS capabilities for (E.164 format). <br><br>
            **Limit:** 1 min

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RcsCapabilitiesResult
            Returns RCS capabilities for each requested phone number.

            The response is a map where keys are phone numbers and values are either:
            - `null` if the number doesn't support RCS
            - An object containing supported cards and actions if RCS is supported

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.rcs.get_capabilities(
            phone_numbers=["+12345678901", "+19876543210"],
        )
        """
        _response = self._raw_client.get_capabilities(phone_numbers=phone_numbers, request_options=request_options)
        return _response.data

    def whitelist(
        self, *, agent_id: str, phone_number: str, request_options: typing.Optional[RequestOptions] = None
    ) -> RcsWhitelistResponse:
        """
        Whitelist a phone number for testing with your test RCS agent.

        ## Overview
        During development and testing, RCS agents can only send messages to whitelisted phone numbers.
        Use this endpoint to whitelist specific phone numbers to send and receive messages from the test agent.

        ## Verification Process
        After whitelisting a number, you'll need to complete verification:

        1. Check the test device for message from "RBM Tester Management"
        2. Click the "Make me a tester" button
        3. Enter the separate 4-digit verification SMS code in the Pinnacle dashboard at:
           ```
           https://app.pinnacle.sh/dashboard/brands/{brandId}?campaignId={campaignId}&campaignType=RCS
           ```

         > **⚠️ Important: Re-whitelisting Numbers**
        >
        > If you whitelist a number that's already whitelisted, you'll receive a new message from "RBM Tester Management". **You must click the "Make me a tester" button again to continue sending and receiving messages.**

        > **Important Notes**
        >
        > - **Verification required:** Messages cannot be sent nor received until you have clicked the "Make me a tester" button on the test device.
        > - **Testing only:** This is only required for test agents. Production agents can message any RCS-enabled number.
        > - **Network limitations:** Whitelisting may be temporarily unavailable for some carriers but are usually restored shortly.

        Parameters
        ----------
        agent_id : str
            The RCS agent ID (must be prefixed with 'agent_')

        phone_number : str
            Phone number to whitelist for testing (E.164 format)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RcsWhitelistResponse
            Phone number successfully whitelisted

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.rcs.whitelist(
            agent_id="agent_XXXXXXXXXXXX",
            phone_number="+12345678901",
        )
        """
        _response = self._raw_client.whitelist(
            agent_id=agent_id, phone_number=phone_number, request_options=request_options
        )
        return _response.data

    def get_link(
        self,
        *,
        agent_id: str,
        test_mode: typing.Optional[bool] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        body: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RcsLinkResult:
        """
        Generate a link for initiating an RCS conversation with your agent.

        Users can click these links to start conversations with your RCS agent directly
        from websites, emails, or other applications.

        Parameters
        ----------
        agent_id : str
            The RCS agent ID (must be prefixed with 'agent_')

        test_mode : typing.Optional[bool]
            Link to the test agent or the production agent if false

        phone_number : typing.Optional[str]
            Fallback phone number (E.164 format) to use if the phone number does not support RCS. If not provided, no url will be generated.

        body : typing.Optional[str]
            Optional message body to pre-fill after the user clicks the link

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RcsLinkResult
            Response containing the generated RCS service id and/or url.

            You can always create a link with the service id that is returned using the format:
            ```
            sms://{PHONE_NUMBER}?service_id={SERVICE_ID}&body={URL_ENCODED_BODY}
            ```

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.rcs.get_link(
            agent_id="agent_XXXXXXXXXXXX",
            test_mode=False,
            phone_number="+12345678901",
            body="Hello, I need help with my order",
        )
        """
        _response = self._raw_client.get_link(
            agent_id=agent_id,
            test_mode=test_mode,
            phone_number=phone_number,
            body=body,
            request_options=request_options,
        )
        return _response.data


class AsyncRcsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRcsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawRcsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRcsClient
        """
        return self._raw_client

    async def get_capabilities(
        self, *, phone_numbers: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> RcsCapabilitiesResult:
        """
        Check RCS capabilities for one or more phone numbers.

        This endpoint allows you to verify which RCS features (cards, buttons, etc.) are supported
        on specific phone numbers before sending RCS messages to them.

        Parameters
        ----------
        phone_numbers : typing.Sequence[str]
            List of phone numbers to check RCS capabilities for (E.164 format). <br><br>
            **Limit:** 1 min

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RcsCapabilitiesResult
            Returns RCS capabilities for each requested phone number.

            The response is a map where keys are phone numbers and values are either:
            - `null` if the number doesn't support RCS
            - An object containing supported cards and actions if RCS is supported

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.rcs.get_capabilities(
                phone_numbers=["+12345678901", "+19876543210"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_capabilities(
            phone_numbers=phone_numbers, request_options=request_options
        )
        return _response.data

    async def whitelist(
        self, *, agent_id: str, phone_number: str, request_options: typing.Optional[RequestOptions] = None
    ) -> RcsWhitelistResponse:
        """
        Whitelist a phone number for testing with your test RCS agent.

        ## Overview
        During development and testing, RCS agents can only send messages to whitelisted phone numbers.
        Use this endpoint to whitelist specific phone numbers to send and receive messages from the test agent.

        ## Verification Process
        After whitelisting a number, you'll need to complete verification:

        1. Check the test device for message from "RBM Tester Management"
        2. Click the "Make me a tester" button
        3. Enter the separate 4-digit verification SMS code in the Pinnacle dashboard at:
           ```
           https://app.pinnacle.sh/dashboard/brands/{brandId}?campaignId={campaignId}&campaignType=RCS
           ```

         > **⚠️ Important: Re-whitelisting Numbers**
        >
        > If you whitelist a number that's already whitelisted, you'll receive a new message from "RBM Tester Management". **You must click the "Make me a tester" button again to continue sending and receiving messages.**

        > **Important Notes**
        >
        > - **Verification required:** Messages cannot be sent nor received until you have clicked the "Make me a tester" button on the test device.
        > - **Testing only:** This is only required for test agents. Production agents can message any RCS-enabled number.
        > - **Network limitations:** Whitelisting may be temporarily unavailable for some carriers but are usually restored shortly.

        Parameters
        ----------
        agent_id : str
            The RCS agent ID (must be prefixed with 'agent_')

        phone_number : str
            Phone number to whitelist for testing (E.164 format)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RcsWhitelistResponse
            Phone number successfully whitelisted

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.rcs.whitelist(
                agent_id="agent_XXXXXXXXXXXX",
                phone_number="+12345678901",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.whitelist(
            agent_id=agent_id, phone_number=phone_number, request_options=request_options
        )
        return _response.data

    async def get_link(
        self,
        *,
        agent_id: str,
        test_mode: typing.Optional[bool] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        body: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RcsLinkResult:
        """
        Generate a link for initiating an RCS conversation with your agent.

        Users can click these links to start conversations with your RCS agent directly
        from websites, emails, or other applications.

        Parameters
        ----------
        agent_id : str
            The RCS agent ID (must be prefixed with 'agent_')

        test_mode : typing.Optional[bool]
            Link to the test agent or the production agent if false

        phone_number : typing.Optional[str]
            Fallback phone number (E.164 format) to use if the phone number does not support RCS. If not provided, no url will be generated.

        body : typing.Optional[str]
            Optional message body to pre-fill after the user clicks the link

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RcsLinkResult
            Response containing the generated RCS service id and/or url.

            You can always create a link with the service id that is returned using the format:
            ```
            sms://{PHONE_NUMBER}?service_id={SERVICE_ID}&body={URL_ENCODED_BODY}
            ```

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.rcs.get_link(
                agent_id="agent_XXXXXXXXXXXX",
                test_mode=False,
                phone_number="+12345678901",
                body="Hello, I need help with my order",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_link(
            agent_id=agent_id,
            test_mode=test_mode,
            phone_number=phone_number,
            body=body,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawToolsClient, RawToolsClient

if typing.TYPE_CHECKING:
    from .contact_card.client import AsyncContactCardClient, ContactCardClient
    from .file.client import AsyncFileClient, FileClient
    from .url.client import AsyncUrlClient, UrlClient


class ToolsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawToolsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._url: typing.Optional[UrlClient] = None
        self._file: typing.Optional[FileClient] = None
        self._contact_card: typing.Optional[ContactCardClient] = None

    @property
    def with_raw_response(self) -> RawToolsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawToolsClient
        """
        return self._raw_client

    @property
    def url(self):
        if self._url is None:
            from .url.client import UrlClient  # noqa: E402

            self._url = UrlClient(client_wrapper=self._client_wrapper)
        return self._url

    @property
    def file(self):
        if self._file is None:
            from .file.client import FileClient  # noqa: E402

            self._file = FileClient(client_wrapper=self._client_wrapper)
        return self._file

    @property
    def contact_card(self):
        if self._contact_card is None:
            from .contact_card.client import ContactCardClient  # noqa: E402

            self._contact_card = ContactCardClient(client_wrapper=self._client_wrapper)
        return self._contact_card


class AsyncToolsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawToolsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._url: typing.Optional[AsyncUrlClient] = None
        self._file: typing.Optional[AsyncFileClient] = None
        self._contact_card: typing.Optional[AsyncContactCardClient] = None

    @property
    def with_raw_response(self) -> AsyncRawToolsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawToolsClient
        """
        return self._raw_client

    @property
    def url(self):
        if self._url is None:
            from .url.client import AsyncUrlClient  # noqa: E402

            self._url = AsyncUrlClient(client_wrapper=self._client_wrapper)
        return self._url

    @property
    def file(self):
        if self._file is None:
            from .file.client import AsyncFileClient  # noqa: E402

            self._file = AsyncFileClient(client_wrapper=self._client_wrapper)
        return self._file

    @property
    def contact_card(self):
        if self._contact_card is None:
            from .contact_card.client import AsyncContactCardClient  # noqa: E402

            self._contact_card = AsyncContactCardClient(client_wrapper=self._client_wrapper)
        return self._contact_card



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.contact import Contact
from ..types.contact_id import ContactId
from ..types.error import Error
from ..types.updated_contact_id import UpdatedContactId

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawContactsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        *,
        id: typing.Optional[str] = None,
        phone_number: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Contact]:
        """
        Retrieve contact information for a given number.

        Parameters
        ----------
        id : typing.Optional[str]
            Unique identifier of a specific contact you want to retrieve. <br>

            Either this parameter or `phoneNumber` must be provided, but not both.

        phone_number : typing.Optional[str]
            Phone number you want to look up contact information for, provided in URL-encoded E.164 format with %2B prefix instead of +.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Contact]
            Successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "contacts",
            method="GET",
            params={
                "id": id,
                "phoneNumber": phone_number,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Contact,
                    parse_obj_as(
                        type_=Contact,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        phone_number: str,
        description: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ContactId]:
        """
        Create a new contact for a given phone number.

        Parameters
        ----------
        phone_number : str
            Phone number to save for your contact, in E.164 format.

        description : typing.Optional[str]
            New notes or details for your contact.

        email : typing.Optional[str]
            New email address for your contact.

        name : typing.Optional[str]
            New name for your contact.

        tags : typing.Optional[typing.Sequence[str]]
            New tags for your contact.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ContactId]
            Successfully created the contact.
        """
        _response = self._client_wrapper.httpx_client.request(
            "contacts",
            method="POST",
            json={
                "phoneNumber": phone_number,
                "description": description,
                "email": email,
                "name": name,
                "tags": tags,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ContactId,
                    parse_obj_as(
                        type_=ContactId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        *,
        id: str,
        description: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdatedContactId]:
        """
        Update an existing contact.

        Parameters
        ----------
        id : str
            ID of the contact you want to update. This identifier is a string that always begins with the prefix `co_`, for example: `co_1234567890`.

        description : typing.Optional[str]
            New notes or details for your contact.

        email : typing.Optional[str]
            New email address for your contact.

        name : typing.Optional[str]
            New name for your contact.

        tags : typing.Optional[typing.Sequence[str]]
            New tags for your contact.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdatedContactId]
            Successfully updated the contact.
        """
        _response = self._client_wrapper.httpx_client.request(
            "contacts",
            method="PUT",
            json={
                "id": id,
                "description": description,
                "email": email,
                "name": name,
                "tags": tags,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdatedContactId,
                    parse_obj_as(
                        type_=UpdatedContactId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawContactsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        *,
        id: typing.Optional[str] = None,
        phone_number: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Contact]:
        """
        Retrieve contact information for a given number.

        Parameters
        ----------
        id : typing.Optional[str]
            Unique identifier of a specific contact you want to retrieve. <br>

            Either this parameter or `phoneNumber` must be provided, but not both.

        phone_number : typing.Optional[str]
            Phone number you want to look up contact information for, provided in URL-encoded E.164 format with %2B prefix instead of +.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Contact]
            Successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "contacts",
            method="GET",
            params={
                "id": id,
                "phoneNumber": phone_number,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Contact,
                    parse_obj_as(
                        type_=Contact,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        phone_number: str,
        description: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ContactId]:
        """
        Create a new contact for a given phone number.

        Parameters
        ----------
        phone_number : str
            Phone number to save for your contact, in E.164 format.

        description : typing.Optional[str]
            New notes or details for your contact.

        email : typing.Optional[str]
            New email address for your contact.

        name : typing.Optional[str]
            New name for your contact.

        tags : typing.Optional[typing.Sequence[str]]
            New tags for your contact.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ContactId]
            Successfully created the contact.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "contacts",
            method="POST",
            json={
                "phoneNumber": phone_number,
                "description": description,
                "email": email,
                "name": name,
                "tags": tags,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ContactId,
                    parse_obj_as(
                        type_=ContactId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        *,
        id: str,
        description: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdatedContactId]:
        """
        Update an existing contact.

        Parameters
        ----------
        id : str
            ID of the contact you want to update. This identifier is a string that always begins with the prefix `co_`, for example: `co_1234567890`.

        description : typing.Optional[str]
            New notes or details for your contact.

        email : typing.Optional[str]
            New email address for your contact.

        name : typing.Optional[str]
            New name for your contact.

        tags : typing.Optional[typing.Sequence[str]]
            New tags for your contact.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdatedContactId]
            Successfully updated the contact.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "contacts",
            method="PUT",
            json={
                "id": id,
                "description": description,
                "email": email,
                "name": name,
                "tags": tags,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdatedContactId,
                    parse_obj_as(
                        type_=UpdatedContactId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .rcs_campaign_schema_extra_agent_emails_item import RcsCampaignSchemaExtraAgentEmailsItem
from .rcs_campaign_schema_extra_agent_phones_item import RcsCampaignSchemaExtraAgentPhonesItem
from .rcs_campaign_schema_extra_agent_websites_item import RcsCampaignSchemaExtraAgentWebsitesItem


class RcsCampaignSchemaExtraAgent(UniversalBaseModel):
    """
    Agent configured to the campaign.
    """

    color: typing.Optional[str] = pydantic.Field(default=None)
    """
    Color of the agent's buttons.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the agent.
    """

    emails: typing.List[RcsCampaignSchemaExtraAgentEmailsItem] = pydantic.Field()
    """
    List of emails.
    """

    hero_url: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="heroUrl")] = pydantic.Field(
        default=None
    )
    """
    Link to the agent's hero. Hero must be an image file with dimensions 1440x448px.
    """

    icon_url: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="iconUrl")] = pydantic.Field(
        default=None
    )
    """
    Link to the agent's icon. Icon must be an image file with dimensions 224x224px.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the agent.
    """

    phones: typing.List[RcsCampaignSchemaExtraAgentPhonesItem] = pydantic.Field()
    """
    List of phone numbers.
    """

    websites: typing.List[RcsCampaignSchemaExtraAgentWebsitesItem] = pydantic.Field()
    """
    List of urls.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawStatusClient, RawStatusClient

if typing.TYPE_CHECKING:
    from .get.client import AsyncGetClient, GetClient


class StatusClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawStatusClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._get: typing.Optional[GetClient] = None

    @property
    def with_raw_response(self) -> RawStatusClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawStatusClient
        """
        return self._raw_client

    @property
    def get(self):
        if self._get is None:
            from .get.client import GetClient  # noqa: E402

            self._get = GetClient(client_wrapper=self._client_wrapper)
        return self._get


class AsyncStatusClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawStatusClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._get: typing.Optional[AsyncGetClient] = None

    @property
    def with_raw_response(self) -> AsyncRawStatusClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawStatusClient
        """
        return self._raw_client

    @property
    def get(self):
        if self._get is None:
            from .get.client import AsyncGetClient  # noqa: E402

            self._get = AsyncGetClient(client_wrapper=self._client_wrapper)
        return self._get



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        RcsAgent,
        RcsAgentEmail,
        RcsAgentPhone,
        RcsAgentWebsite,
        RcsAutofillResponse,
        RcsCampaignHelpKeywords,
        RcsCampaignKeywords,
        RcsCampaignOptInKeywords,
        RcsCampaignOptOutKeywords,
        RcsCampaignTraffic,
        RcsLinks,
        RcsUseCase,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "RcsAgent": ".types",
    "RcsAgentEmail": ".types",
    "RcsAgentPhone": ".types",
    "RcsAgentWebsite": ".types",
    "RcsAutofillResponse": ".types",
    "RcsCampaignHelpKeywords": ".types",
    "RcsCampaignKeywords": ".types",
    "RcsCampaignOptInKeywords": ".types",
    "RcsCampaignOptOutKeywords": ".types",
    "RcsCampaignTraffic": ".types",
    "RcsLinks": ".types",
    "RcsUseCase": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "RcsAgent",
    "RcsAgentEmail",
    "RcsAgentPhone",
    "RcsAgentWebsite",
    "RcsAutofillResponse",
    "RcsCampaignHelpKeywords",
    "RcsCampaignKeywords",
    "RcsCampaignOptInKeywords",
    "RcsCampaignOptOutKeywords",
    "RcsCampaignTraffic",
    "RcsLinks",
    "RcsUseCase",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .message import Message
from .message_with_reaction_direction import MessageWithReactionDirection


class MessageWithReaction(Message):
    """
    Message with additional conversation-specific fields including direction and reaction.
    """

    direction: MessageWithReactionDirection = pydantic.Field()
    """
    Direction of the message flow.
    """

    reaction: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unicode emoji reaction attached to this message. <br>
    
    Null indicates no reaction has been added.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

BrandStatusEnum = typing.Union[
    typing.Literal["PENDING", "UNVERIFIED", "VERIFIED", "VETTED", "FAILED", "INCOMPLETE"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class RcsCampaignHelpKeywords(UniversalBaseModel):
    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Message sent when a user sends HELP. Must include at least one support contact method (phone, email, or website). See the [Keyword Response Messages](/guides/campaigns/rcs-compliance#keyword-response-messages) section for requirements.
    """

    keywords: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Keywords that trigger help response (HELP, SUPPORT, INFO, etc).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .button_clicked_data_button import ButtonClickedDataButton


class ButtonClickedData(UniversalBaseModel):
    """
    Unified data structure for button click events.
    """

    button: ButtonClickedDataButton = pydantic.Field()
    """
    Information about the button that was clicked.
    """

    message_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="messageId")] = pydantic.Field(
        default=None
    )
    """
    ID of the message this button was sent in, or null if not available. To get the message details, use the [GET /messages/{id}](/api-reference/messages/get) endpoint.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class RcsButtonTrigger(UniversalBaseModel):
    """
    Button that sends custom data back to your application when tapped by the recipient.
    """

    metadata: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional additional data to attach to this button.
    """

    payload: str = pydantic.Field()
    """
    Custom data sent to your webhook when the button is tapped. Use this to identify the user's choice.
    """

    title: str = pydantic.Field()
    """
    Display text for the button.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .message_volume_enum import MessageVolumeEnum
from .toll_free_campaign_schema_keywords import TollFreeCampaignSchemaKeywords
from .toll_free_campaign_schema_links import TollFreeCampaignSchemaLinks
from .toll_free_campaign_schema_opt_in import TollFreeCampaignSchemaOptIn
from .toll_free_campaign_schema_options import TollFreeCampaignSchemaOptions
from .toll_free_campaign_schema_use_case import TollFreeCampaignSchemaUseCase


class TollFreeCampaign(UniversalBaseModel):
    campaign_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="campaignId")] = pydantic.Field(
        default=None
    )
    """
    Unique identifier for the campaign. Must begin with the prefix `tf_`.
    """

    keywords: typing.Optional[TollFreeCampaignSchemaKeywords] = pydantic.Field(default=None)
    """
    Keyword response configuration.
    """

    links: typing.Optional[TollFreeCampaignSchemaLinks] = pydantic.Field(default=None)
    """
    Legal documentation links.
    """

    monthly_volume: typing_extensions.Annotated[
        typing.Optional[MessageVolumeEnum], FieldMetadata(alias="monthlyVolume")
    ] = None
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Display name of the campaign. This is not sent to carriers for approval and is only used for your reference on the Pinnacle dashboard.
    """

    opt_in: typing_extensions.Annotated[typing.Optional[TollFreeCampaignSchemaOptIn], FieldMetadata(alias="optIn")] = (
        pydantic.Field(default=None)
    )
    """
    Opt-in method and workflow.
    """

    options: typing.Optional[TollFreeCampaignSchemaOptions] = pydantic.Field(default=None)
    """
    Campaign configuration options.
    """

    production_message_content: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="productionMessageContent")
    ] = pydantic.Field(default=None)
    """
    Example message(s) that would be sent in production. Should reflect the actual content users will receive. See the [Production Message Content](/guides/campaigns/opt-in-compliance#production-message-content) section for requirements.
    """

    use_case: typing_extensions.Annotated[
        typing.Optional[TollFreeCampaignSchemaUseCase], FieldMetadata(alias="useCase")
    ] = pydantic.Field(default=None)
    """
    Use case classification for the campaign.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .message_event_content import MessageEventContent
from .message_event_conversation import MessageEventConversation
from .message_event_direction import MessageEventDirection
from .message_status_enum import MessageStatusEnum
from .webhook_event_enum import WebhookEventEnum


class MessageEvent(UniversalBaseModel):
    """
    Represents an incoming message or message status update received via webhook.
    """

    type: WebhookEventEnum = pydantic.Field()
    """
    Type of webhook event. MESSAGE.STATUS for message status updates or MESSAGE.RECEIVED for inbound messages.
    """

    conversation: MessageEventConversation = pydantic.Field()
    """
    Conversation metadata containing the conversation ID, sender, and recipient information.
    """

    status: MessageStatusEnum
    direction: MessageEventDirection = pydantic.Field()
    """
    Direction of the message flow.
    """

    segments: int = pydantic.Field()
    """
    Number of segments for this message.
    """

    sent_at: typing_extensions.Annotated[str, FieldMetadata(alias="sentAt")] = pydantic.Field()
    """
    Timestamp when the message was sent in ISO 8601 format.
    """

    delivered_at: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="deliveredAt")] = (
        pydantic.Field(default=None)
    )
    """
    Timestamp when the message was delivered in ISO 8601 format.
    Null if not yet delivered or for inbound messages.
    """

    message: MessageEventContent

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .mms_validation_response_segments_value_item import MmsValidationResponseSegmentsValueItem


class MmsValidationResponseSegments(UniversalBaseModel):
    """
    Details about how the MMS content would be segmented for delivery.
    """

    count: int = pydantic.Field()
    """
    Number of segments this message would be split into.
    """

    unsupported_files: typing_extensions.Annotated[typing.List[str], FieldMetadata(alias="unsupportedFiles")] = (
        pydantic.Field()
    )
    """
    List of unsupported media files included in the message. <br>
    
    This may be because the file is unreachable or not supported by the MMS protocol. See [supported file types](https://app.pinnacle.sh/supported-file-types?type=MMS).
    """

    value: typing.List[MmsValidationResponseSegmentsValueItem] = pydantic.Field()
    """
    Message segments with its content breakdown.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PinnacleUrlConfig(UniversalBaseModel):
    to: str = pydantic.Field()
    """
    Destination URL that your shortened link redirects to.
    """

    expires_at: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="expiresAt")] = pydantic.Field(
        default=None
    )
    """
    Expiration date for your shortened link in ISO 8601 format, or null if it's permanent.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import mimetypes
from pathlib import Path
from typing import Optional
import httpx
from ...tools.file.client import FileClient, AsyncFileClient
from ...tools.file.types.upload_file_options import UploadFileOptions
from ...errors.not_found_error import NotFoundError
from ...errors.bad_request_error import BadRequestError
from ...errors.internal_server_error import InternalServerError
from ...types.error import Error
from ...core.client_wrapper import SyncClientWrapper, AsyncClientWrapper


def _get_mime_type(file_path: str) -> str:
    mime_type, _ = mimetypes.guess_type(file_path)

    if not mime_type:
        return "application/octet-stream"

    supported_types = {
        "audio/mpeg",
        "audio/mp4",
        "audio/ogg",
        "audio/aac",
        "audio/webm",
        "audio/wav",
        "audio/3gpp",
        "audio/amr",
        "video/mp4",
        "video/mpeg",
        "video/quicktime",
        "video/webm",
        "video/3gpp",
        "video/H264",
        "video/x-m4v",
        "image/jpeg",
        "image/png",
        "image/gif",
        "image/bmp",
        "image/tiff",
        "image/webp",
        "application/pdf",
        "text/csv",
        "application/rtf",
        "text/vcard",
        "text/calendar",
    }

    if mime_type in supported_types:
        return mime_type

    base_type = mime_type.split("/")[0]
    if base_type in ["audio", "video", "image"]:
        print(
            f"WARNING: MIME type {mime_type} may not be fully supported. Proceeding anyway."
        )
        return mime_type

    return "application/octet-stream"


class FileUploader(FileClient):
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        super().__init__(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper

    def upload_from_path(
        self,
        file_path: str,
        *,
        name: Optional[str] = None,
        options: Optional[UploadFileOptions] = None,
    ) -> str:
        path = Path(file_path)

        if not path.exists():
            raise NotFoundError(body=Error(error=f"File not found: {file_path}"))

        if path.is_dir():
            raise BadRequestError(body=f"Path is a directory, not a file: {file_path}")

        try:
            stats = path.stat()
        except Exception as e:
            raise InternalServerError(
                body=Error(error=f"Failed to access file: {str(e)}")
            )

        size = stats.st_size
        file_name = name or path.name
        content_type = _get_mime_type(file_path)

        upload_result = self.upload(
            content_type=content_type,
            size=size,
            name=file_name,
            options=options or UploadFileOptions(),
        )

        if upload_result.upload_url:
            with open(file_path, "rb") as f:
                file_content = f.read()

            with httpx.Client() as client:
                response = client.put(
                    upload_result.upload_url,
                    content=file_content,
                    headers={"Content-Type": content_type},
                )
                response.raise_for_status()

        return upload_result.download_url


class AsyncFileUploader(AsyncFileClient):
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        super().__init__(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper

    async def upload_from_path(
        self,
        file_path: str,
        *,
        name: Optional[str] = None,
        options: Optional[UploadFileOptions] = None,
    ) -> str:
        path = Path(file_path)

        if not path.exists():
            raise NotFoundError(body=Error(error=f"File not found: {file_path}"))

        if path.is_dir():
            raise BadRequestError(body=f"Path is a directory, not a file: {file_path}")

        try:
            stats = path.stat()
        except Exception as e:
            raise InternalServerError(
                body=Error(error=f"Failed to access file: {str(e)}")
            )

        size = stats.st_size
        file_name = name or path.name
        content_type = _get_mime_type(file_path)

        upload_result = await self.upload(
            content_type=content_type,
            size=size,
            name=file_name,
            options=options or UploadFileOptions(),
        )

        if upload_result.upload_url:
            with open(file_path, "rb") as f:
                file_content = f.read()

            async with httpx.AsyncClient() as client:
                response = await client.put(
                    upload_result.upload_url,
                    content=file_content,
                    headers={"Content-Type": content_type},
                )
                response.raise_for_status()

        return upload_result.download_url



# This file was auto-generated by Fern from our API Definition.

import typing

SubUseCaseEnum = typing.Union[
    typing.Literal[
        "ACCOUNT_NOTIFICATION",
        "CUSTOMER_CARE",
        "DELIVERY_NOTIFICATION",
        "FRAUD_ALERT",
        "HIGHER_EDUCATION",
        "MARKETING",
        "POLLING_VOTING",
        "PUBLIC_SERVICE_ANNOUNCEMENT",
        "SECURITY_ALERT",
        "TWO_FA",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import httpx
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .environment import PinnacleEnvironment

if typing.TYPE_CHECKING:
    from .audiences.client import AsyncAudiencesClient, AudiencesClient
    from .brands.client import AsyncBrandsClient, BrandsClient
    from .campaigns.client import AsyncCampaignsClient, CampaignsClient
    from .contacts.client import AsyncContactsClient, ContactsClient
    from .conversations.client import AsyncConversationsClient, ConversationsClient
    from .messages.client import AsyncMessagesClient, MessagesClient
    from .phone_numbers.client import AsyncPhoneNumbersClient, PhoneNumbersClient
    from .rcs.client import AsyncRcsClient, RcsClient
    from .status.client import AsyncStatusClient, StatusClient
    from .tools.client import AsyncToolsClient, ToolsClient
    from .webhooks.client import AsyncWebhooksClient, WebhooksClient


class PinnacleBase:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : PinnacleEnvironment
        The environment to use for requests from the client. from .environment import PinnacleEnvironment



        Defaults to PinnacleEnvironment.DEFAULT



    api_key : str
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from rcs import Pinnacle

    client = Pinnacle(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: PinnacleEnvironment = PinnacleEnvironment.DEFAULT,
        api_key: str,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._brands: typing.Optional[BrandsClient] = None
        self._audiences: typing.Optional[AudiencesClient] = None
        self._contacts: typing.Optional[ContactsClient] = None
        self._conversations: typing.Optional[ConversationsClient] = None
        self._messages: typing.Optional[MessagesClient] = None
        self._phone_numbers: typing.Optional[PhoneNumbersClient] = None
        self._rcs: typing.Optional[RcsClient] = None
        self._webhooks: typing.Optional[WebhooksClient] = None
        self._campaigns: typing.Optional[CampaignsClient] = None
        self._status: typing.Optional[StatusClient] = None
        self._tools: typing.Optional[ToolsClient] = None

    @property
    def brands(self):
        if self._brands is None:
            from .brands.client import BrandsClient  # noqa: E402

            self._brands = BrandsClient(client_wrapper=self._client_wrapper)
        return self._brands

    @property
    def audiences(self):
        if self._audiences is None:
            from .audiences.client import AudiencesClient  # noqa: E402

            self._audiences = AudiencesClient(client_wrapper=self._client_wrapper)
        return self._audiences

    @property
    def contacts(self):
        if self._contacts is None:
            from .contacts.client import ContactsClient  # noqa: E402

            self._contacts = ContactsClient(client_wrapper=self._client_wrapper)
        return self._contacts

    @property
    def conversations(self):
        if self._conversations is None:
            from .conversations.client import ConversationsClient  # noqa: E402

            self._conversations = ConversationsClient(client_wrapper=self._client_wrapper)
        return self._conversations

    @property
    def messages(self):
        if self._messages is None:
            from .messages.client import MessagesClient  # noqa: E402

            self._messages = MessagesClient(client_wrapper=self._client_wrapper)
        return self._messages

    @property
    def phone_numbers(self):
        if self._phone_numbers is None:
            from .phone_numbers.client import PhoneNumbersClient  # noqa: E402

            self._phone_numbers = PhoneNumbersClient(client_wrapper=self._client_wrapper)
        return self._phone_numbers

    @property
    def rcs(self):
        if self._rcs is None:
            from .rcs.client import RcsClient  # noqa: E402

            self._rcs = RcsClient(client_wrapper=self._client_wrapper)
        return self._rcs

    @property
    def webhooks(self):
        if self._webhooks is None:
            from .webhooks.client import WebhooksClient  # noqa: E402

            self._webhooks = WebhooksClient(client_wrapper=self._client_wrapper)
        return self._webhooks

    @property
    def campaigns(self):
        if self._campaigns is None:
            from .campaigns.client import CampaignsClient  # noqa: E402

            self._campaigns = CampaignsClient(client_wrapper=self._client_wrapper)
        return self._campaigns

    @property
    def status(self):
        if self._status is None:
            from .status.client import StatusClient  # noqa: E402

            self._status = StatusClient(client_wrapper=self._client_wrapper)
        return self._status

    @property
    def tools(self):
        if self._tools is None:
            from .tools.client import ToolsClient  # noqa: E402

            self._tools = ToolsClient(client_wrapper=self._client_wrapper)
        return self._tools


class AsyncPinnacleBase:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : PinnacleEnvironment
        The environment to use for requests from the client. from .environment import PinnacleEnvironment



        Defaults to PinnacleEnvironment.DEFAULT



    api_key : str
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from rcs import AsyncPinnacle

    client = AsyncPinnacle(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: PinnacleEnvironment = PinnacleEnvironment.DEFAULT,
        api_key: str,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._brands: typing.Optional[AsyncBrandsClient] = None
        self._audiences: typing.Optional[AsyncAudiencesClient] = None
        self._contacts: typing.Optional[AsyncContactsClient] = None
        self._conversations: typing.Optional[AsyncConversationsClient] = None
        self._messages: typing.Optional[AsyncMessagesClient] = None
        self._phone_numbers: typing.Optional[AsyncPhoneNumbersClient] = None
        self._rcs: typing.Optional[AsyncRcsClient] = None
        self._webhooks: typing.Optional[AsyncWebhooksClient] = None
        self._campaigns: typing.Optional[AsyncCampaignsClient] = None
        self._status: typing.Optional[AsyncStatusClient] = None
        self._tools: typing.Optional[AsyncToolsClient] = None

    @property
    def brands(self):
        if self._brands is None:
            from .brands.client import AsyncBrandsClient  # noqa: E402

            self._brands = AsyncBrandsClient(client_wrapper=self._client_wrapper)
        return self._brands

    @property
    def audiences(self):
        if self._audiences is None:
            from .audiences.client import AsyncAudiencesClient  # noqa: E402

            self._audiences = AsyncAudiencesClient(client_wrapper=self._client_wrapper)
        return self._audiences

    @property
    def contacts(self):
        if self._contacts is None:
            from .contacts.client import AsyncContactsClient  # noqa: E402

            self._contacts = AsyncContactsClient(client_wrapper=self._client_wrapper)
        return self._contacts

    @property
    def conversations(self):
        if self._conversations is None:
            from .conversations.client import AsyncConversationsClient  # noqa: E402

            self._conversations = AsyncConversationsClient(client_wrapper=self._client_wrapper)
        return self._conversations

    @property
    def messages(self):
        if self._messages is None:
            from .messages.client import AsyncMessagesClient  # noqa: E402

            self._messages = AsyncMessagesClient(client_wrapper=self._client_wrapper)
        return self._messages

    @property
    def phone_numbers(self):
        if self._phone_numbers is None:
            from .phone_numbers.client import AsyncPhoneNumbersClient  # noqa: E402

            self._phone_numbers = AsyncPhoneNumbersClient(client_wrapper=self._client_wrapper)
        return self._phone_numbers

    @property
    def rcs(self):
        if self._rcs is None:
            from .rcs.client import AsyncRcsClient  # noqa: E402

            self._rcs = AsyncRcsClient(client_wrapper=self._client_wrapper)
        return self._rcs

    @property
    def webhooks(self):
        if self._webhooks is None:
            from .webhooks.client import AsyncWebhooksClient  # noqa: E402

            self._webhooks = AsyncWebhooksClient(client_wrapper=self._client_wrapper)
        return self._webhooks

    @property
    def campaigns(self):
        if self._campaigns is None:
            from .campaigns.client import AsyncCampaignsClient  # noqa: E402

            self._campaigns = AsyncCampaignsClient(client_wrapper=self._client_wrapper)
        return self._campaigns

    @property
    def status(self):
        if self._status is None:
            from .status.client import AsyncStatusClient  # noqa: E402

            self._status = AsyncStatusClient(client_wrapper=self._client_wrapper)
        return self._status

    @property
    def tools(self):
        if self._tools is None:
            from .tools.client import AsyncToolsClient  # noqa: E402

            self._tools = AsyncToolsClient(client_wrapper=self._client_wrapper)
        return self._tools


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: PinnacleEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .dlc_with_extended_brand_and_status_keywords_help import DlcWithExtendedBrandAndStatusKeywordsHelp
from .dlc_with_extended_brand_and_status_keywords_opt_in import DlcWithExtendedBrandAndStatusKeywordsOptIn
from .dlc_with_extended_brand_and_status_keywords_opt_out import DlcWithExtendedBrandAndStatusKeywordsOptOut


class DlcWithExtendedBrandAndStatusKeywords(UniversalBaseModel):
    """
    Keyword response configuration.
    """

    help: typing_extensions.Annotated[DlcWithExtendedBrandAndStatusKeywordsHelp, FieldMetadata(alias="HELP")] = (
        pydantic.Field()
    )
    """
    Help keyword settings.
    """

    opt_in: typing_extensions.Annotated[DlcWithExtendedBrandAndStatusKeywordsOptIn, FieldMetadata(alias="OPT_IN")] = (
        pydantic.Field()
    )
    """
    Opt-in keyword settings.
    """

    opt_out: typing_extensions.Annotated[
        DlcWithExtendedBrandAndStatusKeywordsOptOut, FieldMetadata(alias="OPT_OUT")
    ] = pydantic.Field()
    """
    Opt-out keyword settings.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.create_url_options import CreateUrlOptions
from ...types.shortened_url import ShortenedUrl
from ...types.shortened_url_with_click_data import ShortenedUrlWithClickData
from .raw_client import AsyncRawUrlClient, RawUrlClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UrlClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUrlClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUrlClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUrlClient
        """
        return self._raw_client

    def create(
        self,
        *,
        url: str,
        options: typing.Optional[CreateUrlOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ShortenedUrl:
        """
        Create a shortened URL that redirects visitors to the provided destination URL.

        Parameters
        ----------
        url : str
            Destination URL for the shortened link that visitors will be redirected to when clicked.

        options : typing.Optional[CreateUrlOptions]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ShortenedUrl
            Successfully created.

        Examples
        --------
        from rcs import CreateUrlOptions, Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.tools.url.create(
            url="https://www.pinnacle.sh/",
            options=CreateUrlOptions(
                expires_at="2025-06-23T16:18:25.000Z",
            ),
        )
        """
        _response = self._raw_client.create(url=url, options=options, request_options=request_options)
        return _response.data

    def get(
        self, link_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ShortenedUrlWithClickData:
        """
        Retrieve configuration and details for your shortened URL using its unique identifier.

        Parameters
        ----------
        link_id : str
            Unique identifier from your shortened URL. For example, for `https://pncl.to/ePzVxILF`, the `linkId` is `ePzVxILF`. <br>

            See the response of [Create Shortened URL](./create-url) for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ShortenedUrlWithClickData
            Returns your URL details.

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.tools.url.get(
            link_id="ePzVxILF",
        )
        """
        _response = self._raw_client.get(link_id, request_options=request_options)
        return _response.data

    def update(
        self,
        link_id: str,
        *,
        url: typing.Optional[str] = OMIT,
        options: typing.Optional[CreateUrlOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ShortenedUrl:
        """
        Update the destination or expiration date of an existing shortened URL. Expiring links cannot be updated into a permalink.

        Parameters
        ----------
        link_id : str
            Unique identifier from your shortened URL. For example, for `https://pncl.to/ePzVxILF`, the `linkId` is `ePzVxILF`. <br>

            See the response of [Create Shortened URL](./create-url) for more information.

        url : typing.Optional[str]
            New destination URL where your visitors will be redirected.

        options : typing.Optional[CreateUrlOptions]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ShortenedUrl
            Returns the updated shortened URL and its configuration.

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.tools.url.update(
            link_id="ePzVxILF",
            url="https://www.pinnacle.sh/",
        )
        """
        _response = self._raw_client.update(link_id, url=url, options=options, request_options=request_options)
        return _response.data


class AsyncUrlClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUrlClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUrlClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUrlClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        url: str,
        options: typing.Optional[CreateUrlOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ShortenedUrl:
        """
        Create a shortened URL that redirects visitors to the provided destination URL.

        Parameters
        ----------
        url : str
            Destination URL for the shortened link that visitors will be redirected to when clicked.

        options : typing.Optional[CreateUrlOptions]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ShortenedUrl
            Successfully created.

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle, CreateUrlOptions

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.url.create(
                url="https://www.pinnacle.sh/",
                options=CreateUrlOptions(
                    expires_at="2025-06-23T16:18:25.000Z",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(url=url, options=options, request_options=request_options)
        return _response.data

    async def get(
        self, link_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ShortenedUrlWithClickData:
        """
        Retrieve configuration and details for your shortened URL using its unique identifier.

        Parameters
        ----------
        link_id : str
            Unique identifier from your shortened URL. For example, for `https://pncl.to/ePzVxILF`, the `linkId` is `ePzVxILF`. <br>

            See the response of [Create Shortened URL](./create-url) for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ShortenedUrlWithClickData
            Returns your URL details.

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.url.get(
                link_id="ePzVxILF",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(link_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        link_id: str,
        *,
        url: typing.Optional[str] = OMIT,
        options: typing.Optional[CreateUrlOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ShortenedUrl:
        """
        Update the destination or expiration date of an existing shortened URL. Expiring links cannot be updated into a permalink.

        Parameters
        ----------
        link_id : str
            Unique identifier from your shortened URL. For example, for `https://pncl.to/ePzVxILF`, the `linkId` is `ePzVxILF`. <br>

            See the response of [Create Shortened URL](./create-url) for more information.

        url : typing.Optional[str]
            New destination URL where your visitors will be redirected.

        options : typing.Optional[CreateUrlOptions]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ShortenedUrl
            Returns the updated shortened URL and its configuration.

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.url.update(
                link_id="ePzVxILF",
                url="https://www.pinnacle.sh/",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(link_id, url=url, options=options, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.datetime_utils import serialize_datetime
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.campaign_enum import CampaignEnum
from ..types.conversation import Conversation
from ..types.conversation_list import ConversationList
from ..types.error import Error
from ..types.get_conversation_params import GetConversationParams
from ..types.message_list import MessageList
from ..types.sender import Sender
from ..types.successful_conversation_update import SuccessfulConversationUpdate
from .types.conversations_list_messages_request_direction import ConversationsListMessagesRequestDirection
from .types.conversations_list_messages_request_sort_order import ConversationsListMessagesRequestSortOrder
from .types.conversations_list_messages_request_status import ConversationsListMessagesRequestStatus
from .types.conversations_list_messages_request_type import ConversationsListMessagesRequestType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawConversationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, *, request: GetConversationParams, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Optional[Conversation]]:
        """
        Fetch a specific conversation using either its unique identifier or by matching sender and recipient details.

        Parameters
        ----------
        request : GetConversationParams

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Optional[Conversation]]
            Returns the full conversation details, or null if not found.
        """
        _response = self._client_wrapper.httpx_client.request(
            "conversations/get",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=GetConversationParams, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[Conversation],
                    parse_obj_as(
                        type_=typing.Optional[Conversation],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list(
        self,
        *,
        page_index: int,
        brand_id: typing.Optional[str] = OMIT,
        campaign_id: typing.Optional[str] = OMIT,
        campaign_type: typing.Optional[CampaignEnum] = OMIT,
        page_size: typing.Optional[int] = OMIT,
        receiver: typing.Optional[str] = OMIT,
        sender: typing.Optional[Sender] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ConversationList]:
        """
        Retrieves conversations by page with optional filtering based off provided parameters.

        Parameters
        ----------
        page_index : int
            Zero-based index for pagination.

        brand_id : typing.Optional[str]
            The unique identifier of the brand to filter conversations. This identifier is a string that always begins with the prefix `b_`, for example: `b_1234567890`.

        campaign_id : typing.Optional[str]
            The unique identifier of the campaign to filter conversations. This identifier is a string that begins with the prefix:
            - TOLL_FREE: `tf_` (e.g., `tf_1234567890`)
            - 10DLC: `dlc_` (e.g., `dlc_1234567890`)
            - RCS: `rcs_` (e.g., `rcs_1234567890`)

        campaign_type : typing.Optional[CampaignEnum]

        page_size : typing.Optional[int]
            Number of conversations to return per page.

        receiver : typing.Optional[str]
            Receiver's phone number in E.164 format.

        sender : typing.Optional[Sender]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConversationList]
            Returns conversations that meets your filters.
        """
        _response = self._client_wrapper.httpx_client.request(
            "conversations/list",
            method="POST",
            json={
                "brandId": brand_id,
                "campaignId": campaign_id,
                "campaignType": campaign_type,
                "pageIndex": page_index,
                "pageSize": page_size,
                "receiver": receiver,
                "sender": sender,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationList,
                    parse_obj_as(
                        type_=ConversationList,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self, *, id: str, notes: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SuccessfulConversationUpdate]:
        """
        Update the notes associated with a specific conversation.

        Parameters
        ----------
        id : str
            The unique identifier of the conversation to update. This identifier is a string that always begins with the prefix `conv_`, for example: `conv_1234567890`.

        notes : str
            New notes or comments for the conversation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SuccessfulConversationUpdate]
            Successfully updated the conversation.
        """
        _response = self._client_wrapper.httpx_client.request(
            "conversations/update",
            method="POST",
            json={
                "id": id,
                "notes": notes,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SuccessfulConversationUpdate,
                    parse_obj_as(
                        type_=SuccessfulConversationUpdate,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_messages(
        self,
        id: str,
        *,
        page_index: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        sort_order: typing.Optional[ConversationsListMessagesRequestSortOrder] = None,
        direction: typing.Optional[ConversationsListMessagesRequestDirection] = None,
        status: typing.Optional[ConversationsListMessagesRequestStatus] = None,
        type: typing.Optional[ConversationsListMessagesRequestType] = None,
        date_from: typing.Optional[dt.datetime] = None,
        date_to: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[MessageList]:
        """
        Retrieve a paginated and filtered list of messages for a specific conversation.

        Parameters
        ----------
        id : str
            Unique identifier of the conversation. This identifier is a string that always begins with the prefix `conv_`, for example: `conv_1234567890`.

        page_index : typing.Optional[int]
            Zero-based index for pagination.

        page_size : typing.Optional[int]
            Number of messages to return per page.

        sort_order : typing.Optional[ConversationsListMessagesRequestSortOrder]
            Sort order for messages. <br>

            - `asc`: Oldest messages first
            - `desc`: Newest messages first (default)

        direction : typing.Optional[ConversationsListMessagesRequestDirection]
            Filter messages by direction. <br>

            - `INBOUND`: Messages received from contacts
            - `OUTBOUND`: Messages sent to contacts

        status : typing.Optional[ConversationsListMessagesRequestStatus]
            Filter messages by delivery status.

        type : typing.Optional[ConversationsListMessagesRequestType]
            Filter messages by protocol type.

        date_from : typing.Optional[dt.datetime]
            Filter messages sent on or after this date (ISO 8601 format).

        date_to : typing.Optional[dt.datetime]
            Filter messages sent on or before this date (ISO 8601 format).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[MessageList]
            Returns a paginated list of messages in the conversation.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/messages",
            method="POST",
            params={
                "pageIndex": page_index,
                "pageSize": page_size,
                "sortOrder": sort_order,
                "direction": direction,
                "status": status,
                "type": type,
                "dateFrom": serialize_datetime(date_from) if date_from is not None else None,
                "dateTo": serialize_datetime(date_to) if date_to is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MessageList,
                    parse_obj_as(
                        type_=MessageList,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawConversationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, *, request: GetConversationParams, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Optional[Conversation]]:
        """
        Fetch a specific conversation using either its unique identifier or by matching sender and recipient details.

        Parameters
        ----------
        request : GetConversationParams

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Optional[Conversation]]
            Returns the full conversation details, or null if not found.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "conversations/get",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=GetConversationParams, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[Conversation],
                    parse_obj_as(
                        type_=typing.Optional[Conversation],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list(
        self,
        *,
        page_index: int,
        brand_id: typing.Optional[str] = OMIT,
        campaign_id: typing.Optional[str] = OMIT,
        campaign_type: typing.Optional[CampaignEnum] = OMIT,
        page_size: typing.Optional[int] = OMIT,
        receiver: typing.Optional[str] = OMIT,
        sender: typing.Optional[Sender] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ConversationList]:
        """
        Retrieves conversations by page with optional filtering based off provided parameters.

        Parameters
        ----------
        page_index : int
            Zero-based index for pagination.

        brand_id : typing.Optional[str]
            The unique identifier of the brand to filter conversations. This identifier is a string that always begins with the prefix `b_`, for example: `b_1234567890`.

        campaign_id : typing.Optional[str]
            The unique identifier of the campaign to filter conversations. This identifier is a string that begins with the prefix:
            - TOLL_FREE: `tf_` (e.g., `tf_1234567890`)
            - 10DLC: `dlc_` (e.g., `dlc_1234567890`)
            - RCS: `rcs_` (e.g., `rcs_1234567890`)

        campaign_type : typing.Optional[CampaignEnum]

        page_size : typing.Optional[int]
            Number of conversations to return per page.

        receiver : typing.Optional[str]
            Receiver's phone number in E.164 format.

        sender : typing.Optional[Sender]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConversationList]
            Returns conversations that meets your filters.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "conversations/list",
            method="POST",
            json={
                "brandId": brand_id,
                "campaignId": campaign_id,
                "campaignType": campaign_type,
                "pageIndex": page_index,
                "pageSize": page_size,
                "receiver": receiver,
                "sender": sender,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationList,
                    parse_obj_as(
                        type_=ConversationList,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self, *, id: str, notes: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SuccessfulConversationUpdate]:
        """
        Update the notes associated with a specific conversation.

        Parameters
        ----------
        id : str
            The unique identifier of the conversation to update. This identifier is a string that always begins with the prefix `conv_`, for example: `conv_1234567890`.

        notes : str
            New notes or comments for the conversation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SuccessfulConversationUpdate]
            Successfully updated the conversation.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "conversations/update",
            method="POST",
            json={
                "id": id,
                "notes": notes,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SuccessfulConversationUpdate,
                    parse_obj_as(
                        type_=SuccessfulConversationUpdate,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_messages(
        self,
        id: str,
        *,
        page_index: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        sort_order: typing.Optional[ConversationsListMessagesRequestSortOrder] = None,
        direction: typing.Optional[ConversationsListMessagesRequestDirection] = None,
        status: typing.Optional[ConversationsListMessagesRequestStatus] = None,
        type: typing.Optional[ConversationsListMessagesRequestType] = None,
        date_from: typing.Optional[dt.datetime] = None,
        date_to: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[MessageList]:
        """
        Retrieve a paginated and filtered list of messages for a specific conversation.

        Parameters
        ----------
        id : str
            Unique identifier of the conversation. This identifier is a string that always begins with the prefix `conv_`, for example: `conv_1234567890`.

        page_index : typing.Optional[int]
            Zero-based index for pagination.

        page_size : typing.Optional[int]
            Number of messages to return per page.

        sort_order : typing.Optional[ConversationsListMessagesRequestSortOrder]
            Sort order for messages. <br>

            - `asc`: Oldest messages first
            - `desc`: Newest messages first (default)

        direction : typing.Optional[ConversationsListMessagesRequestDirection]
            Filter messages by direction. <br>

            - `INBOUND`: Messages received from contacts
            - `OUTBOUND`: Messages sent to contacts

        status : typing.Optional[ConversationsListMessagesRequestStatus]
            Filter messages by delivery status.

        type : typing.Optional[ConversationsListMessagesRequestType]
            Filter messages by protocol type.

        date_from : typing.Optional[dt.datetime]
            Filter messages sent on or after this date (ISO 8601 format).

        date_to : typing.Optional[dt.datetime]
            Filter messages sent on or before this date (ISO 8601 format).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[MessageList]
            Returns a paginated list of messages in the conversation.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/messages",
            method="POST",
            params={
                "pageIndex": page_index,
                "pageSize": page_size,
                "sortOrder": sort_order,
                "direction": direction,
                "status": status,
                "type": type,
                "dateFrom": serialize_datetime(date_from) if date_from is not None else None,
                "dateTo": serialize_datetime(date_to) if date_to is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MessageList,
                    parse_obj_as(
                        type_=MessageList,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

CompanyEntityTypeEnum = typing.Union[
    typing.Literal["LLC", "SOLE_PROPRIETORSHIP", "PARTNERSHIP", "CORP", "S_CORP"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .rcs_agent import RcsAgent
    from .rcs_agent_email import RcsAgentEmail
    from .rcs_agent_phone import RcsAgentPhone
    from .rcs_agent_website import RcsAgentWebsite
    from .rcs_autofill_response import RcsAutofillResponse
    from .rcs_campaign_help_keywords import RcsCampaignHelpKeywords
    from .rcs_campaign_keywords import RcsCampaignKeywords
    from .rcs_campaign_opt_in_keywords import RcsCampaignOptInKeywords
    from .rcs_campaign_opt_out_keywords import RcsCampaignOptOutKeywords
    from .rcs_campaign_traffic import RcsCampaignTraffic
    from .rcs_links import RcsLinks
    from .rcs_use_case import RcsUseCase
_dynamic_imports: typing.Dict[str, str] = {
    "RcsAgent": ".rcs_agent",
    "RcsAgentEmail": ".rcs_agent_email",
    "RcsAgentPhone": ".rcs_agent_phone",
    "RcsAgentWebsite": ".rcs_agent_website",
    "RcsAutofillResponse": ".rcs_autofill_response",
    "RcsCampaignHelpKeywords": ".rcs_campaign_help_keywords",
    "RcsCampaignKeywords": ".rcs_campaign_keywords",
    "RcsCampaignOptInKeywords": ".rcs_campaign_opt_in_keywords",
    "RcsCampaignOptOutKeywords": ".rcs_campaign_opt_out_keywords",
    "RcsCampaignTraffic": ".rcs_campaign_traffic",
    "RcsLinks": ".rcs_links",
    "RcsUseCase": ".rcs_use_case",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "RcsAgent",
    "RcsAgentEmail",
    "RcsAgentPhone",
    "RcsAgentWebsite",
    "RcsAutofillResponse",
    "RcsCampaignHelpKeywords",
    "RcsCampaignKeywords",
    "RcsCampaignOptInKeywords",
    "RcsCampaignOptOutKeywords",
    "RcsCampaignTraffic",
    "RcsLinks",
    "RcsUseCase",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .campaign_query import CampaignQuery
from .conversation_contact import ConversationContact
from .conversation_sender import ConversationSender


class Conversation(UniversalBaseModel):
    """
    Conversation that was found.
    """

    brand_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="brandId")] = pydantic.Field(
        default=None
    )
    """
    The unique identifier of the brand associated with this conversation. This identifier is a string that always begins with the prefix `b_`, for example: `b_1234567890`.
    """

    campaign: typing.Optional[CampaignQuery] = pydantic.Field(default=None)
    """
    Campaign information if this conversation is part of a marketing campaign. This is an object that contains the campaign ID and type.
    """

    contact: ConversationContact = pydantic.Field()
    """
    Contact information for the recipient in a conversation.
    """

    created_at: typing_extensions.Annotated[str, FieldMetadata(alias="createdAt")] = pydantic.Field()
    """
    ISO 8601 timestamp when the conversation was created.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the conversation. This identifier is a string that always begins with the prefix `conv_`, for example: `conv_1234567890`.
    """

    notes: str = pydantic.Field()
    """
    Free-form notes or comments about the conversation.
    """

    sender: typing.Optional[ConversationSender] = pydantic.Field(default=None)
    """
    The sender of messages in this conversation. Can be:
    - A phone number with its capabilities and metadata
    - An RCS agent with ID and name
    - `null` if no sender is associated
    """

    updated_at: typing_extensions.Annotated[str, FieldMetadata(alias="updatedAt")] = pydantic.Field()
    """
    ISO 8601 timestamp when the conversation was last updated.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ZodError(UniversalBaseModel):
    description: str = pydantic.Field()
    """
    Human-readable summary of validation failures.
    """

    errors: typing.Dict[str, typing.Optional[typing.Any]] = pydantic.Field()
    """
    Structured dictionary of issues containing two main sections:
    - `errors`: Array of global validation errors not tied to specific fields
    - `properties`: Object mapping field names to their specific validation errors, where each field contains an `errors` array
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class MessageSchedule(UniversalBaseModel):
    """
    Define when and how your message should be sent.
    """

    send_at: typing_extensions.Annotated[str, FieldMetadata(alias="sendAt")] = pydantic.Field()
    """
    The date and time you want your message to be sent. 
    - Use [ISO 8601 format](https://en.wikipedia.org/wiki/ISO_8601), for example: `2024-12-17T14:30:00Z`. Format: `YYYY-MM-DDThh:mm` (e.g., `2024-12-25T14:30`). 
    - This time must be at least 5 minutes in the future.
    - The message will be scheduled based on the `timezone` you provide.
    - If you set a `recurrence` schedule, this is the start date and time for the recurring schedule.
    """

    recurrence: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS cron expression for recurring schedules (6 fields). <br>
    
    [Learn more about cron expressions](https://docs.aws.amazon.com/eventbridge/latest/userguide/eb-scheduled-rule-pattern.html). <br>
    
    **Examples:**
    - `0 10 * * ? *` - Every day at 10:00 AM
    - `0/30 * * * ? *` - Every 30 minutes
    - `0 9 ? * MON-FRI *` - Every weekday at 9:00 AM
    - `0 12 1 * ? *` - First day of every month at noon
    """

    timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    IANA timezone identifier (e.g., `America/New_York`, `UTC`). <br>
    
    Defaults to `UTC` if not specified.
    """

    end_date: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="endDate")] = pydantic.Field(
        default=None
    )
    """
    Date and time when recurring messages should stop. <br>
    
    Format: `YYYY-MM-DDThh:mm`. Required if `recurrence` is set.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .toll_free_status_enum import TollFreeStatusEnum


class GetTollFreeCampaignStatusResponseUpdates(UniversalBaseModel):
    """
    Status of phone numbers that are being attached to this campaign. <br>

    Note that phone numbers can only be attached to `VERIFIED` campaigns via the [attach campaign endpoint](../phone-numbers/attach-campaign).
    """

    errors: typing.List[str] = pydantic.Field()
    """
    List of errors that occurred.
    """

    number: str = pydantic.Field()
    """
    Number associated with the campaign in E.164 format.
    """

    status: TollFreeStatusEnum = pydantic.Field()
    """
    Current assignment status of the toll free number to this campaign. <br>
    
    `IN_PROGRESS`: Being reviewed.<br>
    `WAITING_FOR_PROVIDER`: Being reviewed by our toll-free vendor.<br>
    `WAITING_FOR_TELECO:` Final review by carriers before number is live.<br>
    `VERIFIED`: Verified and ready to send messages.<br>
    `REJECTED`: Issues were found related to your campaign. See the [errors field](toll-free-campaign#response.body.error).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class Pagination(UniversalBaseModel):
    page: int = pydantic.Field()
    """
    Current page number (1-indexed).
    """

    limit: int = pydantic.Field()
    """
    Items per page (max 100, default 50).
    """

    total: int = pydantic.Field()
    """
    Total number of contacts in the audience.
    """

    has_more: typing_extensions.Annotated[bool, FieldMetadata(alias="hasMore")] = pydantic.Field()
    """
    Whether more pages exist.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
